
if SERVER then
	AddCSLuaFile("shared.lua")
end

if CLIENT then

	SWEP.PrintName		= "SWEP Construction Kit"
	SWEP.Author			= "Clavus"
	SWEP.Contact		= "clavus@clavusstudios.com"
	SWEP.Purpose		= "Design SWEP ironsights and clientside models"
	SWEP.Instructions	= "Click a mouse button for the menu. You'll figure it out from there."
	SWEP.Slot			= 5
	SWEP.SlotPos		= 10
	SWEP.ViewModelFlip	= true

end

SWEP.HoldType = "pistol"
SWEP.HoldTypes = { "normal", "melee", "melee2", "fist", 
"knife", "smg", "ar2", "pistol", "rpg", "physgun", 
"grenade", "shotgun", "crossbow", "slam" }

SWEP.Spawnable			= true
SWEP.AdminSpawnable		= true

SWEP.ViewModel			= "models/weapons/v_pist_p228.mdl"
SWEP.WorldModel			= "models/weapons/w_pist_p228.mdl"

SWEP.ViewModelFOV		= 80
SWEP.BobScale			= 0
SWEP.SwayScale			= 0

SWEP.Primary.Automatic	= false

SWEP.IronsightTime = 0.2

SWEP.IronSightsPos = Vector (0, 0, 0)
SWEP.IronSightsAng = Vector (0, 0, 0)

function SWEP:Initialize()
	
	self:SetWeaponHoldType(self.HoldType)
	
	self:SetIronSights( true )
	self:ResetIronSights()
	
end

function SWEP:PrimaryAttack()

	self.Weapon:SetNextPrimaryFire(CurTime() + 0.2)
	
	if CLIENT then
		self:OpenMenu()
	end
	if SinglePlayer() then
		self.Owner:SendLua("LocalPlayer():GetActiveWeapon():OpenMenu()")
	end
	
end

function SWEP:SecondaryAttack()
	
	self.Weapon:SetNextSecondaryFire(CurTime() + 0.2)
	
	if CLIENT then
		self:OpenMenu()
	end
	if SinglePlayer() then
		self.Owner:SendLua("LocalPlayer():GetActiveWeapon():OpenMenu()")
	end
	
end


function SWEP:SetupDataTables()

	self:DTVar( "Bool", 0, "ironsights" )
	self:DTVar( "Bool", 1, "thirdperson" )
	
end

function SWEP:ToggleIronSights()
	self.dt.ironsights = !self.dt.ironsights
end

function SWEP:SetIronSights( b )
	self.dt.ironsights = b
end

function SWEP:GetIronSights()
	return self.dt.ironsights
end

function SWEP:ResetIronSights()
	RunConsoleCommand("_sp_ironsight_x", 0)
	RunConsoleCommand("_sp_ironsight_y", 0)
	RunConsoleCommand("_sp_ironsight_z", 0)
	RunConsoleCommand("_sp_ironsight_pitch", 0)
	RunConsoleCommand("_sp_ironsight_yaw", 0)
	RunConsoleCommand("_sp_ironsight_roll", 0)
end

function SWEP:ToggleThirdPerson()
	self.dt.thirdperson = !self.dt.thirdperson
	if (self.dt.thirdperson) then self.Owner:CrosshairDisable()
	else self.Owner:CrosshairEnable() end
end

function SWEP:SetThirdPerson( b )
	self.dt.thirdperson = b
	if (self.dt.thirdperson) then self.Owner:CrosshairDisable()
	else self.Owner:CrosshairEnable() end
end

function SWEP:GetThirdPerson()
	return self.dt.thirdperson
end

function SWEP:GetViewModelPosition(pos, ang)

	local bIron = self.dt.ironsights
	local fIronTime = self.fIronTime or 0

	if (not bIron and fIronTime < CurTime() - self.IronsightTime) then
		return pos, ang
	end
	
	self.IronSightsPos, self.IronSightsAng = self:GetIronSightCoordination()

	local Mul = 1.0

	if (fIronTime > CurTime() - self.IronsightTime) then
		Mul = math.Clamp((CurTime() - fIronTime) / self.IronsightTime, 0, 1)

		if not bIron then Mul = 1 - Mul end
	end

	local Offset	= self.IronSightsPos

	if (self.IronSightsAng) then
		ang = ang * 1
		ang:RotateAroundAxis(ang:Right(), 		self.IronSightsAng.x * Mul)
		ang:RotateAroundAxis(ang:Up(), 		self.IronSightsAng.y * Mul)
		ang:RotateAroundAxis(ang:Forward(), 	self.IronSightsAng.z * Mul)
	end

	local Right 	= ang:Right()
	local Up 		= ang:Up()
	local Forward 	= ang:Forward()

	pos = pos + Offset.x * Right * Mul
	pos = pos + Offset.y * Forward * Mul
	pos = pos + Offset.z * Up * Mul
	
	return pos, ang
end

local ir_x = CreateConVar( "_sp_ironsight_x", 0.0 )
local ir_y = CreateConVar( "_sp_ironsight_y", 0.0 )
local ir_z = CreateConVar( "_sp_ironsight_z", 0.0 )
local ir_p = CreateConVar( "_sp_ironsight_pitch", 0.0 )
local ir_yw = CreateConVar( "_sp_ironsight_yaw", 0.0 )
local ir_r = CreateConVar( "_sp_ironsight_roll", 0.0 )

function SWEP:GetIronSightCoordination()
	
	local vec = Vector( ir_x:GetFloat(), ir_y:GetFloat(), ir_z:GetFloat() )
	local ang = Vector( ir_p:GetFloat(), ir_yw:GetFloat(), ir_r:GetFloat() )
	return vec, ang
	
end

function SWEP:GetHoldTypes()
	return self.HoldTypes
end

/***************************
	Helper functions
***************************/
local function GetSWEP( pl )
	local wep = pl:GetActiveWeapon()
	if (ValidEntity(wep) and wep:GetClass() == "swep_construction_kit") then
		return wep
	end
	Error("Not holding SWEP Construction Kit!")
	return NULL
end
	
/***********************************************
		SERVER CODE YEEEEEAAAAAHHH
***********************************************/
if SERVER then
	
	local function Cmd_SetHoldType( pl, cmd, args )

		local holdtype = args[1]
		local wep = GetSWEP( pl )
		if (holdtype and table.HasValue( wep:GetHoldTypes(), holdtype )) then
			wep:SetWeaponHoldType( holdtype )
			wep.HoldType = holdtype
		end	

	end
	concommand.Add("swepck_setholdtype", Cmd_SetHoldType)

	local function Cmd_ToggleThirdPerson( pl, cmd, args )

		local wep = GetSWEP( pl )
		wep:ToggleThirdPerson()

	end
	concommand.Add("swepck_togglethirdperson", Cmd_ToggleThirdPerson)
	
	local function Cmd_PlayAnimation( pl, cmd, args )

		local wep = GetSWEP( pl )
		wep:SendWeaponAnim( tonumber(args[1] or 0) )

	end
	concommand.Add("swepck_playanimation", Cmd_PlayAnimation)
	
	local function Cmd_PlayAnimation( pl, cmd, args )

		local wep = GetSWEP( pl )
		wep:ToggleIronSights()

	end
	concommand.Add("swepck_toggleironsights", Cmd_PlayAnimation)

	local function Cmd_ViewModelFOV( pl, cmd, args )

		local wep = GetSWEP( pl )
		wep.ViewModelFOV = tonumber(args[1] or wep.ViewModelFOV)

	end
	concommand.Add("swepck_viewmodelfov", Cmd_ViewModelFOV)
	
	local function Cmd_ViewModel( pl, cmd, args )

		local wep = GetSWEP( pl )
		local newmod = args[1] or wep.ViewModel
		newmod = newmod..".mdl"
		if !file.Exists ("../"..newmod) then return end
		
		util.PrecacheModel(newmod)
		wep.ViewModel = newmod
		pl:SendLua([[LocalPlayer():GetActiveWeapon().ViewModel = "]]..newmod..[["]])
		//pl:SendLua([[LocalPlayer():GetViewModel():SetModel("]]..newmod..[[")]])
		pl:SendLua([[LocalPlayer():GetViewModel():SetWeaponModel("]]..newmod..[[", Entity(]]..wep:EntIndex()..[[))]])
		
		local quickswitch = nil
		for k, v in pairs( pl:GetWeapons() ) do
			if (v:GetClass() != wep:GetClass()) then 
				quickswitch = v:GetClass()
				break
			end
		end
		
		if (quickswitch) then
			pl:SelectWeapon( quickswitch )
			pl:SelectWeapon( wep:GetClass() )
		else
			pl:ChatPrint("Switch weapons to make the new viewmodel show up")
		end
		
		print("Changed viewmodel to \""..wep.ViewModel.."\"")
		
	end
	concommand.Add("swepck_viewmodel", Cmd_ViewModel)
	
	local function Cmd_WorldModel( pl, cmd, args )

		local wep = GetSWEP( pl )
		local newmod = args[1] or wep.WorldModel
		newmod = newmod..".mdl"
		if !file.Exists ("../"..newmod) then return end
		
		util.PrecacheModel(newmod)
		wep.WorldModel = newmod
		wep:SetModel(newmod)
		pl:SendLua([[LocalPlayer():GetActiveWeapon().WorldModel = "]]..newmod..[["]])
		pl:SendLua([[LocalPlayer():GetActiveWeapon():SetModel("]]..newmod..[[")]])
		
		print("Changed worldmodel to \""..wep.WorldModel.."\"")
		
	end
	concommand.Add("swepck_worldmodel", Cmd_WorldModel)

end

/***********************************************
		CLIENT CODE WHOOOOOOOOOOO
***********************************************/
if CLIENT then
	
	local useThirdPerson = false
	local thirdPersonAngle = Angle(0,-90,0)
	local mlast_x = ScrW()/2
	local mlast_y = ScrH()/2
	local v_models = {}
	local v_panelCache = {}
	local v_modelListing = nil
	local w_models = {}

	local ir_drag = {
		x = { true, "-x", 25 },
		y = { false, "y", 25 },
		z = { true, "y", 25 },
		pitch = { false, "y", 10 },
		yaw = { false, "x", 10 },
		roll = { false, "y", 10 }
	}
	local drag_modes = {
		["x / z"] = { "x", "z" },
		["y"] = { "y" },
		["pitch / yaw"] = { "pitch", "yaw" },
		["roll"] = { "roll" }	
	}
	local cur_drag_mode = "x / z"
	SWEP.Frame = nil
	
	local function PrintVec( vec )
		local px, py, pz = math.floor(vec.x*1000)/1000,math.floor(vec.y*1000)/1000,math.floor(vec.z*1000)/1000
		return "Vector("..px..", "..py..", "..pz..")"
	end
	
	local function PrintAngle( angle )
		local pp, py, pr = math.floor(angle.p*1000)/1000,math.floor(angle.y*1000)/1000,math.floor(angle.r*1000)/1000
		return "Angle("..pp..", "..py..", "..pr..")"
	end
	
	local function PrintColor( col )
		return "Color("..col.r..", "..col.g..", "..col.b..", "..col.a..")"
	end	
	
	local function GetIronSightPrintText( vec, ang )
		return "SWEP.IronSightsPos = "..PrintVec( vec ).."`\nSWEP.IronSightsAng = "..PrintVec( ang )
	end

	local function GetVModelsText()
		local str = ("SWEP.VElements = {\n")
		for k, v in pairs( v_models ) do
			if (v.type == "Model") then
				str = str.."\t[\""..k.."\"] = { type = \"Model\", model = \""..v.model.."\", bone = \""..v.bone.."\", pos = "..PrintVec(v.pos)
				str = str..", angle = "..PrintAngle( v.angle )..", size = "..PrintVec(v.size)..", color = "..PrintColor( v.color )
				str = str..", surpresslightning = "..tostring(v.surpresslightning)..", material = \""..v.material.."\"}"
			elseif (v.type == "Sprite") then
				str = str.."\t[\""..k.."\"] = { type = \"Sprite\", sprite = \""..v.sprite.."\", bone = \""..v.bone.."\", pos = "..PrintVec(v.pos)
				str = str..", size = { x = "..v.size.x..", y = "..v.size.y.." }, color = "..PrintColor( v.color )..", nocull = "..tostring(v.nocull)
				str = str..", additive = "..tostring(v.additive)..", vertexalpha = "..tostring(v.vertexalpha)..", vertexcolor = "..tostring(v.vertexcolor)
				str = str..", ignorez = "..tostring(v.ignorez).."}"		
			end
			if (k != #v_models) then str = str.."," end
			str = str.."\n"
		end
		str = str.."}"
		return str
	end
	
	local function ClearViewModels()
		v_models = {}
		if (v_modelListing) then v_modelListing:Clear() end
		for k, v in pairs( v_panelCache ) do
			if (v) then 
				v:Remove()
			end
		end
		v_panelCache = {}
	end
	
	function SWEP:Think()

		// Create the clientside models here because Garry says we can't do it in the render hook
		for k, v in pairs( v_models ) do
			if (v.type == "Model" and v.model and (!ValidEntity(v.modelEnt) or string.lower(v.modelEnt:GetModel()) != string.lower(v.model)) and file.Exists ("../"..v.model) ) then
				
				v.modelEnt = ClientsideModel(v.model, RENDER_GROUP_VIEW_MODEL_OPAQUE)
				if (ValidEntity(v.modelEnt)) then
					v.modelEnt:SetPos(self:GetPos())
					v.modelEnt:SetAngles(self:GetAngles())
					v.modelEnt:SetParent(self)
					v.modelEnt:SetNoDraw(true)
				else
					v.modelEnt = nil
				end
				
			elseif (v.type == "Sprite" and v.sprite and (!v.spriteMaterial or v.createdSprite != v.sprite) and file.Exists ("../materials/"..v.sprite..".vmt")) then
				
				local name = v.sprite.."-"
				local params = { ["$basetexture"] = v.sprite }
				if (v.nocull) then 
					params["$nocull"] = 1
					name = name.."1" // make sure we create a unique name based on the selected options
				else name = name.."0" end
				if (v.additive) then 
					params["$additive"] = 1
					name = name.."1"					
				else name = name.."0" end
				if (v.vertexalpha) then 
					params["$vertexalpha"] = 1
					name = name.."1"
				else name = name.."0" end
				if (v.vertexcolor) then 
					params["$vertexcolor"] = 1
					name = name.."1"
				else name = name.."0" end
				if (v.ignorez) then 
					params["$ignorez"] = 1
					name = name.."1"
				else name = name.."0" end
				
				v.createdSprite = v.sprite
				v.spriteMaterial = CreateMaterial(name,"UnlitGeneric",params)
				
			end
		end
		
		/************************
			Camera fiddling
		************************/
		useThirdPerson = self:GetThirdPerson()
		
		local mx, my = gui.MousePos()
		local diffx, diffy = (mx - mlast_x), (my - mlast_y)
		
		if (input.IsMouseDown(MOUSE_RIGHT) and !(diffx > 40 or diffy > 40)) then // right mouse press without sudden jumps
		
			if (useThirdPerson) then
			
				thirdPersonAngle = thirdPersonAngle + Angle( diffy/2, diffx/2, 0 )
				
			elseif (self.Frame and self.Frame:IsVisible()) then
				// ironsight adjustment
				for k, v in pairs( ir_drag ) do
					if (v[1]) then
						local temp = GetConVar( "_sp_ironsight_"..k ):GetFloat()
						if (v[2] == "x") then
							local add = -(diffx/v[3])
							if (self.ViewModelFlip) then add = add*-1 end
							RunConsoleCommand( "_sp_ironsight_"..k, temp + add )
						elseif (v[2] == "-x") then
							local add = diffx/v[3]
							if (self.ViewModelFlip) then add = add*-1 end
							RunConsoleCommand( "_sp_ironsight_"..k, temp + add )
						elseif (v[2] == "y") then
							RunConsoleCommand( "_sp_ironsight_"..k, temp - diffy/v[3] )
						end
					end
				end
			
			end
			
		end
		
		mlast_x, mlast_y = mx, my
	end
	
	/********************************
		All viewmodel drawing magic
	*********************************/
	function SWEP:ViewModelDrawn()
		
		local vm = self.Owner:GetViewModel()
		
		for k, v in pairs( v_models ) do
		
			local model = v.modelEnt
			local sprite = v.spriteMaterial
			
			if (v.type == "Model" and ValidEntity(model)) then
				local m = vm:GetBoneMatrix(vm:LookupBone(v.bone))
				local pos, ang = m:GetTranslation(), m:GetAngle()
				
				if (self.ViewModelFlip) then
					ang.r = -ang.r // Fixes mirrored models
				end
				
				model:SetPos(pos + ang:Forward() * v.pos.x + ang:Right() * v.pos.y + ang:Up() * v.pos.z )
				ang:RotateAroundAxis(ang:Up(), v.angle.y)
				ang:RotateAroundAxis(ang:Right(), v.angle.p)
				ang:RotateAroundAxis(ang:Forward(), v.angle.r)

				model:SetAngles(ang)
				model:SetModelScale(v.size)
				
				if (v.material == "") then
					model:SetMaterial("")
				elseif (model:GetMaterial() != v.material) then
					model:SetMaterial( v.material )
				end
				
				if (v.surpresslightning) then
					render.SuppressEngineLighting(true)
				end
				
				render.SetColorModulation(v.color.r/255, v.color.g/255, v.color.b/255)
				render.SetBlend(v.color.a/255)
				model:DrawModel()
				render.SetBlend(1)
				render.SetColorModulation(1, 1, 1)
				
				if (v.surpresslightning) then
					render.SuppressEngineLighting(false)
				end
				
			elseif (v.type == "Sprite" and sprite) then
				
				local m = vm:GetBoneMatrix(vm:LookupBone(v.bone))
				local pos, ang = m:GetTranslation(), m:GetAngle()
				
				if (self.ViewModelFlip) then
					ang.r = -ang.r // Fixes mirrored models
				end
				
				local drawpos = pos + ang:Forward() * v.pos.x + ang:Right() * v.pos.y + ang:Up() * v.pos.z
				render.SetMaterial(sprite)
				render.DrawSprite(drawpos, v.size.x, v.size.y, v.color)
				
			end
			
		end
		
	end
	
	
	function SWEP:Holster( wep )
		useThirdPerson = false
		return true
	end
	
	function SWEP:DrawHUD()

		local text = ""
		if (useThirdPerson) then
			text = "Hold right mouse and drag to rotate"
		elseif (self.Frame and self.Frame:IsVisible()) then
			text = "Hold right mouse and drag to adjust ironsights (mode: "..cur_drag_mode..")"
		end
		draw.SimpleTextOutlined(text, "default", ScrW()/2, ScrH()/4, Color(255,255,255,255), TEXT_ALIGN_CENTER, TEXT_ALIGN_TOP, 1, Color(20,20,20,255))
		
	end
	
	/***************************
				Menu
	***************************/
	local function CreateMenu()
		
		local wep = GetSWEP( LocalPlayer() )
		
		local f = vgui.Create("DFrame")
		f:SetSize( 300, 580 )
		f:SetPos( ScrW()/4 - 150, ScrH()/2 - 250 )
		f:SetTitle( "SWEP Construction Kit" )
		f:SetDraggable( true )
		f:ShowCloseButton( true )
		f:SetSizable( false )
		f:SetDeleteOnClose( false )

		local tbtn = vgui.Create( "DButton", f )
		tbtn:SetPos( 155, 2 )
		tbtn:SetSize( 120, 18 )
		tbtn:SetText( "Toggle thirdperson" )
		tbtn.DoClick = function()
			RunConsoleCommand("swepck_togglethirdperson")
		end
		
		local tab = vgui.Create( "DPropertySheet", f )
		tab:SetPos( 5, 30 )
		tab:SetSize( f:GetWide() - 10, f:GetTall()-35 )
		
		local pweapon = vgui.Create("DPanel", tab)
		pweapon.Paint = function() surface.SetDrawColor(50,50,50,255) surface.DrawRect(0,0,pweapon:GetWide(),pweapon:GetTall()) end
		local pironsight = vgui.Create("DPanel", tab)
		pironsight.Paint = function() surface.SetDrawColor(50,50,50,255) surface.DrawRect(0,0,pironsight:GetWide(),pironsight:GetTall()) end
		local pmodels = vgui.Create("DPanel", tab)
		pmodels.Paint = function() surface.SetDrawColor(50,50,50,255) surface.DrawRect(0,0,pmodels:GetWide(),pmodels:GetTall()) end
		local pwmodels = vgui.Create("DPanel", tab)
		pwmodels.Paint = function() surface.SetDrawColor(50,50,50,255) surface.DrawRect(0,0,pwmodels:GetWide(),pwmodels:GetTall()) end
		
		tab:AddSheet( "Weapon", pweapon, "gui/silkicons/bomb", false, false, "Modify weapon settings" )
		tab:AddSheet( "Ironsights", pironsight, "gui/silkicons/wrench", false, false, "Modify ironsights" )
		tab:AddSheet( "VModels", pmodels, "gui/silkicons/brick_add", false, false, "Modify view models" )
		tab:AddSheet( "WModels", pwmodels, "gui/silkicons/brick_add", false, false, "Modify world models" )
		
		/*****************
			Weapon page
		*****************/
		local addy = 10
		
		// Weapon model
		local vlabel = vgui.Create( "DLabel", pweapon )
		vlabel:SetPos( 10, addy )
		vlabel:SetSize( 80, 20 )
		vlabel:SetText( "View model:" )
		
		local vtext = vgui.Create( "DTextEntry", pweapon)
		vtext:SetPos( 85, addy )
		vtext:SetSize( 180, 20 )
		vtext:SetMultiline(false)
		vtext:SetText( wep.ViewModel )
		vtext.OnTextChanged = function()
			local newmod = string.gsub(vtext:GetValue(), ".mdl", "")
			RunConsoleCommand("swepck_viewmodel", newmod)
			
			// clear view model additions
			if (newmod != LocalPlayer():GetViewModel():GetModel()) then
				ClearViewModels()
			end
			
		end

		addy = addy + 25
		
		local wlabel = vgui.Create( "DLabel", pweapon )
		wlabel:SetPos( 10, addy )
		wlabel:SetSize( 80, 20 )
		wlabel:SetText( "World model:" )
		
		local wtext = vgui.Create( "DTextEntry", pweapon)
		wtext:SetPos( 85, addy )
		wtext:SetSize( 180, 20 )
		wtext:SetMultiline(false)
		wtext:SetText( wep.WorldModel )
		wtext.OnTextChanged = function()
			local newmod = string.gsub(wtext:GetValue(), ".mdl", "")
			RunConsoleCommand("swepck_worldmodel", newmod)
		end
		wtext:SetEditable( false )
		
		addy = addy + 30
		
		// Weapon hold type
		local hlabel = vgui.Create( "DLabel", pweapon )
		hlabel:SetPos( 10, addy )
		hlabel:SetSize( 150, 20 )
		hlabel:SetText( "Hold type (3rd person):" )
		
		local hbox = vgui.Create( "DMultiChoice", pweapon )
		hbox:SetPos( 140, addy )
		hbox:SetSize( 125, 20 )
		hbox:SetEditable( false )
		hbox:SetText( wep.HoldType )
		for k, v in pairs( wep:GetHoldTypes() ) do
			hbox:AddChoice( v )
		end
		hbox.OnSelect = function(panel,index,value)
			wep:SetWeaponHoldType( value )
			wep.HoldType = value
			RunConsoleCommand("swepck_setholdtype", value)
		end
		
		addy = addy + 30
		
		// Flip viewmodel
		local fcbox = vgui.Create( "DCheckBoxLabel", pweapon )
		fcbox:SetPos( 10, addy )
		fcbox:SetSize( 150, 20 )
		fcbox:SetText( "Flip viewmodel" )
		if (wep.ViewModelFlip) then fcbox:SetValue(1)
		else fcbox:SetValue(0) end
		fcbox.OnChange = function()
			wep.ViewModelFlip = fcbox:GetChecked()
		end
		
		addy = addy + 30
		
		// View model FOV slider
		local fovslider = vgui.Create( "DNumSlider", pweapon )
		fovslider:SetPos( 10, addy )
		fovslider:SetWide( 260 )
		fovslider:SetText( "View model FOV" )
		fovslider:SetMin( 20 )
		fovslider:SetMax( 140 )
		fovslider:SetDecimals( 0 )
		fovslider:SetValue( wep.ViewModelFOV )
		fovslider.OnValueChanged = function( panel, value )
			wep.ViewModelFOV = tonumber(value)
			RunConsoleCommand("swepck_viewmodelfov", value)
		end
		
		addy = addy + 45
		
		// Animations
		local alabel = vgui.Create( "DLabel", pweapon )
		alabel:SetPos( 10, addy )
		alabel:SetSize( 200, 20 )
		alabel:SetText( "Play animation:" )
		
		addy = addy + 25
		
		local aabtn = vgui.Create( "DButton", pweapon )
		aabtn:SetPos( 10, addy )
		aabtn:SetSize( 120, 18 )
		aabtn:SetText( "Primary Attack" )
		aabtn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_PRIMARYATTACK)
			LocalPlayer():SetAnimation(PLAYER_ATTACK1)
		end
		
		local arbtn = vgui.Create( "DButton", pweapon )
		arbtn:SetPos( 140, addy )
		arbtn:SetSize( 120, 18 )
		arbtn:SetText( "Reload" )
		arbtn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_RELOAD)
			LocalPlayer():SetAnimation(PLAYER_RELOAD)
		end
		
		addy = addy + 20
		
		local m1btn = vgui.Create( "DButton", pweapon )
		m1btn:SetPos( 10, addy )
		m1btn:SetSize( 120, 18 )
		m1btn:SetText( "Melee 1" )
		m1btn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_MISSCENTER)
			LocalPlayer():SetAnimation(PLAYER_ATTACK1)
		end
		
		local m2btn = vgui.Create( "DButton", pweapon )
		m2btn:SetPos( 140, addy )
		m2btn:SetSize( 120, 18 )
		m2btn:SetText( "Melee 2" )
		m2btn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_HITCENTER)
			LocalPlayer():SetAnimation(PLAYER_ATTACK1)
		end
		
		addy = addy + 20
		
		local gcbtn = vgui.Create( "DButton", pweapon )
		gcbtn:SetPos( 10, addy )
		gcbtn:SetSize( 120, 18 )
		gcbtn:SetText( "Grenade pull (CSS)" )
		gcbtn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_PULLPIN)
		end
		
		local ghbtn = vgui.Create( "DButton", pweapon )
		ghbtn:SetPos( 140, addy )
		ghbtn:SetSize( 120, 18 )
		ghbtn:SetText( "Grenade pull (HL2)" )
		ghbtn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_PULLBACK_HIGH)
		end
		
		addy = addy + 20
		
		local wdbtn = vgui.Create( "DButton", pweapon )
		wdbtn:SetPos( 10, addy )
		wdbtn:SetSize( 120, 18 )
		wdbtn:SetText( "Draw" )
		wdbtn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_DRAW)
		end
		
		local tgbtn = vgui.Create( "DButton", pweapon )
		tgbtn:SetPos( 140, addy )
		tgbtn:SetSize( 120, 18 )
		tgbtn:SetText( "Throw grenade" )
		tgbtn.DoClick = function()
			RunConsoleCommand("swepck_playanimation", ACT_VM_THROW)
			LocalPlayer():SetAnimation(PLAYER_ATTACK1)
		end
		
		/*********************
			Ironsights page
		*********************/
		addy = 10
		
		local icbox = vgui.Create( "DCheckBoxLabel", pironsight )
		icbox:SetPos( 10, addy )
		icbox:SetSize( 150, 20 )
		icbox:SetText( "Enable ironsights" )
		if (wep:GetIronSights()) then icbox:SetValue(1)
		else icbox:SetValue(0) end
		icbox.OnChange = function()
			if (wep:GetIronSights() != icbox:GetChecked()) then
				RunConsoleCommand("swepck_toggleironsights")
			end
		end
	
		local ribtn = vgui.Create( "DButton", pironsight )
		ribtn:SetPos( 140, addy )
		ribtn:SetSize( 125, 20 )
		ribtn:SetText( "Reset ironsights" )
		ribtn.DoClick = function()
			wep:ResetIronSights()
		end

		addy = addy + 30
		
		local modlabel = vgui.Create( "DLabel", pironsight )
		modlabel:SetPos( 10, addy )
		modlabel:SetSize( 100, 20 )
		modlabel:SetText( "Drag mode:" )
		
		local drbox = vgui.Create( "DMultiChoice", pironsight )
		drbox:SetPos( 140, addy )
		drbox:SetSize( 125, 20 )
		drbox:SetEditable( false )
		drbox:SetText( cur_drag_mode )
		for k, v in pairs( drag_modes ) do
			drbox:AddChoice( k )
		end
		drbox.OnSelect = function(panel,index,value)
			local modes = drag_modes[value]
			cur_drag_mode = value
			for k, v in pairs( ir_drag ) do
				v[1] = table.HasValue( modes, k ) // set the drag modus
			end
		end
		
		addy = addy + 30
		
		local ixslider = vgui.Create( "DNumSlider", pironsight )
		ixslider:SetPos( 10, addy )
		ixslider:SetWide( 260 )
		ixslider:SetText( "Translate x" )
		ixslider:SetMin( -20 )
		ixslider:SetMax( 20 )
		ixslider:SetDecimals( 3 )
		ixslider:SetConVar( "_sp_ironsight_x" )
		
		addy = addy + 45
		
		local iyslider = vgui.Create( "DNumSlider", pironsight )
		iyslider:SetPos( 10, addy )
		iyslider:SetWide( 260 )
		iyslider:SetText( "Translate y" )
		iyslider:SetMin( -20 )
		iyslider:SetMax( 20 )
		iyslider:SetDecimals( 3 )
		iyslider:SetConVar( "_sp_ironsight_y" )
		
		addy = addy + 45
		
		local izslider = vgui.Create( "DNumSlider", pironsight )
		izslider:SetPos( 10, addy )
		izslider:SetWide( 260 )
		izslider:SetText( "Translate z" )
		izslider:SetMin( -20 )
		izslider:SetMax( 20 )
		izslider:SetDecimals( 3 )
		izslider:SetConVar( "_sp_ironsight_z" )
		
		addy = addy + 45
		
		local ipslider = vgui.Create( "DNumSlider", pironsight )
		ipslider:SetPos( 10, addy )
		ipslider:SetWide( 260 )
		ipslider:SetText( "Rotate pitch" )
		ipslider:SetMin( -70 )
		ipslider:SetMax( 70 )
		ipslider:SetDecimals( 3 )
		ipslider:SetConVar( "_sp_ironsight_pitch" )
		
		addy = addy + 45
		
		local iyaslider = vgui.Create( "DNumSlider", pironsight )
		iyaslider:SetPos( 10, addy )
		iyaslider:SetWide( 260 )
		iyaslider:SetText( "Rotate yaw" )
		iyaslider:SetMin( -70 )
		iyaslider:SetMax( 70 )
		iyaslider:SetDecimals( 3 )
		iyaslider:SetConVar( "_sp_ironsight_yaw" )
		
		addy = addy + 45
		
		local irslider = vgui.Create( "DNumSlider", pironsight )
		irslider:SetPos( 10, addy )
		irslider:SetWide( 260 )
		irslider:SetText( "Rotate roll" )
		irslider:SetMin( -70 )
		irslider:SetMax( 70 )
		irslider:SetDecimals( 3 )
		irslider:SetConVar( "_sp_ironsight_roll" )
		
		addy = addy + 45
		
		local prbtn = vgui.Create( "DButton", pironsight )
		prbtn:SetPos( 10, addy )
		prbtn:SetSize( 260, 30 )
		prbtn:SetText( "Print ironsights code to console" )
		prbtn.DoClick = function()
			local vec, ang = wep:GetIronSightCoordination()
			MsgN("*********************************************")
			MsgN(GetIronSightPrintText( vec, ang ))
			MsgN("*********************************************")
			LocalPlayer():ChatPrint("Code printed to console!")
		end
		
		addy = addy + 35
		
		local pcbtn = vgui.Create( "DButton", pironsight )
		pcbtn:SetPos( 10, addy )
		pcbtn:SetSize( 260, 30 )
		pcbtn:SetText( "Copy ironsights code to clipboard" )
		pcbtn.DoClick = function()
			local vec, ang = wep:GetIronSightCoordination()
			SetClipboardText(GetIronSightPrintText( vec, ang ))
			LocalPlayer():ChatPrint("Code copied to clipboard!")
		end
		
		/*********************
			Models page
		*********************/
		addy = 10
		
		local mlist = vgui.Create( "DListView", pmodels)
		v_modelListing = mlist
		
		local mlabel = vgui.Create( "DLabel", pmodels )
		mlabel:SetPos( 10, addy )
		mlabel:SetSize( 80, 20 )
		mlabel:SetText( "New element:" )
		
		function CreateNote( text )
			local templabel = vgui.Create( "DLabel" )
			templabel:SetText( text )
			templabel:SetSize( 140, 20 )

			local notif = vgui.Create( "DNotify" , pmodels )
			notif:SetPos( 120, 10 )
			notif:SetSize( 140, 20 )
			notif:SetLife( 5 )
			notif:AddItem(templabel)
			
		end
		
		addy = addy + 25
		
		local mntext = vgui.Create("DTextEntry", pmodels )
		mntext:SetPos( 10, addy )
		mntext:SetSize( 120, 20 )
		mntext:SetMultiline(false)
		mntext:SetText( "some_unique_name" )
		
		local tpbox = vgui.Create( "DMultiChoice", pmodels )
		tpbox:SetPos( 130, addy )
		tpbox:SetSize( 85, 20 )
		tpbox:SetEditable( false )
		tpbox:SetText( "Model" )
		tpbox:AddChoice( "Model" )
		tpbox:AddChoice( "Sprite" )
		local boxselected = "Model"
		tpbox.OnSelect = function( p, index, value )
			boxselected = value
		end
		
		local mnbtn = vgui.Create( "DButton", pmodels )
		mnbtn:SetPos( 220, addy )
		mnbtn:SetSize( 45, 20 )
		mnbtn:SetText( "Add" )
		mnbtn.DoClick = function()
			local new = string.Trim( mntext:GetValue() )
			if (new) then
				if (new == "") then CreateNote("Empty name field!") return end
				if (v_models[new] != nil) then CreateNote("Name already exists!") return end
				v_models[new] = {}
				mlist:AddLine(new,boxselected)
			end
		end
		
		addy = addy + 30
		
		mlist:SetPos( 10, addy )
		mlist:SetSize( 260, 100 )
		mlist:SetMultiSelect(false)
		mlist:SetDrawBackground(true)
		mlist:AddColumn("Name")
		mlist:AddColumn("Type")
		
		addy = addy + 135
		
		local rmbtn = vgui.Create( "DButton", pmodels )
		rmbtn:SetPos( 10, addy )
		rmbtn:SetSize( 260, 25 )
		rmbtn:SetText( "Remove selected" )
		
		addy = addy + 30
		
		// Populates a DMultiChoice with all the bones of the viewmodel
		// returns if it has a first option
		local function PopulateBoneList( choicelist )
			local viewmodel = LocalPlayer():GetViewModel()
			local hasfirstoption
			for i = 0, viewmodel:GetBoneCount() - 1 do
				choicelist:AddChoice(viewmodel:GetBoneName(i))
				if (!firstoption) then hasfirstoption = true end
			end
			return hasfirstoption
		end
		
		local panely = addy
		
		/*** Model panel for adjusting models ***
		Name:
		Model:
		Bone name:
		Translation x / y / z
		Rotation pitch / yaw / role
		Model size x / y / z
		Material
		Color modulation
		*/
		local function CreateModelPanel( name, x, y )
			
			local data = v_models[name]
			
			// default data
			data.type = "Model"
			data.model = ""
			data.bone = ""
			data.pos = Vector(0,0,0)
			data.angle = Angle(0,0,0)
			data.size = Vector(1,1,1)
			data.color = Color(255,255,255,255)
			data.surpresslightning = false
			data.material = ""
			
			local panel = vgui.Create( "DPanel", pmodels )
			panel:SetPos( x, y )
			panel:SetSize( 260, pmodels:GetTall() - panely - 60 )
			panel:SetPaintBackground( true )
			panel.Paint = function() surface.SetDrawColor( 70, 70, 70, 255 ) surface.DrawRect( 0, 0, panel:GetWide(), panel:GetTall() ) end
			
			local addy = 5
			
			local pnmlabel = vgui.Create( "DLabel", panel )
			pnmlabel:SetPos( 5, addy )
			pnmlabel:SetSize( 80, 20 )
			pnmlabel:SetText( "Name:" )
			
			local pnmtext = vgui.Create( "DTextEntry", panel )
			pnmtext:SetPos( 60, addy )
			pnmtext:SetSize( 190, 20 )
			pnmtext:SetMultiline(false)
			pnmtext:SetText( name )
			pnmtext:SetEditable( false )
			
			addy = addy + 25
			
			/*
			The bones
			*/
			local pmolabel = vgui.Create( "DLabel", panel )
			pmolabel:SetPos( 5, addy )
			pmolabel:SetSize( 80, 20 )
			pmolabel:SetText( "Model:" )
			
			local pmmtext = vgui.Create( "DTextEntry", panel )
			pmmtext:SetPos( 60, addy )
			pmmtext:SetSize( 190, 20 )
			pmmtext:SetMultiline(false)
			pmmtext:SetText( "" )
			pmmtext.OnTextChanged = function()
				local newmod = pmmtext:GetValue()
				if file.Exists ("../"..newmod) then
					util.PrecacheModel(newmod)
					data.model = newmod
				end
			end

			addy = addy + 25
			
			local pbonelabel = vgui.Create( "DLabel", panel )
			pbonelabel:SetPos( 5, addy )
			pbonelabel:SetSize( 80, 20 )
			pbonelabel:SetText( "Bone:" )
			
			local bonebox = vgui.Create( "DMultiChoice", panel )
			bonebox:SetPos( 60, addy )
			bonebox:SetSize( 190, 20 )
			bonebox:SetEditable( false )
			bonebox:SetText( "" )
			bonebox.OnSelect = function( p, index, value )
				data.bone = value
			end
			
			local option = PopulateBoneList( bonebox )
			if (option) then 
				bonebox:ChooseOptionID(1)
			end
			
			addy = addy + 25
			
			// **************  Pos  **************
			
			local trlabel = vgui.Create( "DLabel", panel )
			trlabel:SetPos( 5, addy )
			trlabel:SetSize( 80, 20 )
			trlabel:SetText( "Pos x/y/z:" )
			
			local mxwang = vgui.Create( "DNumberWang", panel )
			mxwang:SetPos( 90, addy )
			mxwang:SetSize( 50, 20 )
			mxwang:SetMin( -20 )
			mxwang:SetMax( 20 )
			mxwang:SetDecimals( 3 )
			mxwang.OnValueChanged = function( p, value ) data.pos.x = tonumber(value) end

			local mywang = vgui.Create( "DNumberWang", panel )
			mywang:SetPos( 145, addy )
			mywang:SetSize( 50, 20 )
			mywang:SetMin( -20 )
			mywang:SetMax( 20 )
			mywang:SetDecimals( 3 )
			mywang.OnValueChanged = function( p, value ) data.pos.y = tonumber(value) end

			local mzwang = vgui.Create( "DNumberWang", panel )
			mzwang:SetPos( 200, addy )
			mzwang:SetSize( 50, 20 )
			mzwang:SetMin( -20 )
			mzwang:SetMax( 20 )
			mzwang:SetDecimals( 3 )
			mzwang.OnValueChanged = function( p, value ) data.pos.z = tonumber(value) end
			
			addy = addy + 25
			
			// **************  Angle  **************
			
			local anlabel = vgui.Create( "DLabel", panel )
			anlabel:SetPos( 5, addy )
			anlabel:SetSize( 80, 20 )
			anlabel:SetText( "Angle p/y/r:" )
			
			local mpitchwang = vgui.Create( "DNumberWang", panel )
			mpitchwang:SetPos( 90, addy )
			mpitchwang:SetSize( 50, 20 )
			mpitchwang:SetMin( -180 )
			mpitchwang:SetMax( 180 )
			mpitchwang:SetDecimals( 3 )
			mpitchwang.OnValueChanged = function( p, value ) data.angle.p = tonumber(value) end
			
			local myawwang = vgui.Create( "DNumberWang", panel )
			myawwang:SetPos( 145, addy )
			myawwang:SetSize( 50, 20 )
			myawwang:SetMin( -180 )
			myawwang:SetMax( 180 )
			myawwang:SetDecimals( 3 )
			myawwang.OnValueChanged = function( p, value ) data.angle.y = tonumber(value) end

			local mrollwang = vgui.Create( "DNumberWang", panel )
			mrollwang:SetPos( 200, addy )
			mrollwang:SetSize( 50, 20 )
			mrollwang:SetMin( -180 )
			mrollwang:SetMax( 180 )
			mrollwang:SetDecimals( 3 )
			mrollwang.OnValueChanged = function( p, value ) data.angle.r = tonumber(value) end
			
			addy = addy + 25
			
			// **************  Size  **************
			
			local sizelabel = vgui.Create( "DLabel", panel )
			sizelabel:SetPos( 5, addy )
			sizelabel:SetSize( 80, 20 )
			sizelabel:SetText( "Size (xyz)/y/z:" )
			
			local msxwang = vgui.Create( "DNumberWang", panel )
			msxwang:SetPos( 90, addy-2 )
			msxwang:SetSize( 52, 24 )
			msxwang:SetMin( 0.01 )
			msxwang:SetMax( 10 )
			msxwang:SetDecimals( 3 )

			local msywang = vgui.Create( "DNumberWang", panel )
			msywang:SetPos( 145, addy )
			msywang:SetSize( 50, 20 )
			msywang:SetMin( 0.01 )
			msywang:SetMax( 10 )
			msywang:SetDecimals( 3 )
			msywang.OnValueChanged = function( p, value ) data.size.y = tonumber(value) end
			
			local mszwang = vgui.Create( "DNumberWang", panel )
			mszwang:SetPos( 200, addy )
			mszwang:SetSize( 50, 20 )
			mszwang:SetMin( 0.01 )
			mszwang:SetMax( 10 )
			mszwang:SetDecimals( 3 )
			mszwang.OnValueChanged = function( p, value ) data.size.z = tonumber(value) end
			
			// make the x numberwang set the total size
			msxwang.OnValueChanged = function( p, value )
				mszwang:SetValue( value )
				msywang:SetValue( value )
				data.size.x = tonumber(value) 
			end
			msxwang:SetValue(0.5)

			addy = addy + 25
			
			local collabel = vgui.Create( "DLabel", panel )
			collabel:SetPos( 5, addy )
			collabel:SetSize( 80, 20 )
			collabel:SetText( "Color r/g/b/a:" )
			
			local colrwang = vgui.Create( "DNumberWang", panel )
			colrwang:SetPos( 90, addy )
			colrwang:SetSize( 38, 20 )
			colrwang:SetMin( 0 )
			colrwang:SetMax( 255 )
			colrwang:SetDecimals( 0 )
			colrwang.OnValueChanged = function( p, value ) data.color.r = tonumber(value) end
			colrwang:SetValue(255)
			
			local colgwang = vgui.Create( "DNumberWang", panel )
			colgwang:SetPos( 130, addy )
			colgwang:SetSize( 38, 20 )
			colgwang:SetMin( 0 )
			colgwang:SetMax( 255 )
			colgwang:SetDecimals( 0 )
			colgwang.OnValueChanged = function( p, value ) data.color.g = tonumber(value) end
			colgwang:SetValue(255)
			
			local colbwang = vgui.Create( "DNumberWang", panel )
			colbwang:SetPos( 170, addy )
			colbwang:SetSize( 38, 20 )
			colbwang:SetMin( 0 )
			colbwang:SetMax( 255 )
			colbwang:SetDecimals( 0 )
			colbwang.OnValueChanged = function( p, value ) data.color.b = tonumber(value) end
			colbwang:SetValue(255)
			
			local colawang = vgui.Create( "DNumberWang", panel )
			colawang:SetPos( 210, addy )
			colawang:SetSize( 40, 20 )
			colawang:SetMin( 0 )
			colawang:SetMax( 255 )
			colawang:SetDecimals( 0 )
			colawang.OnValueChanged = function( p, value ) data.color.a = tonumber(value) end
			colawang:SetValue(255)
			
			addy = addy + 25
			
			local lschbox = vgui.Create( "DCheckBoxLabel", panel )
			lschbox:SetPos( 5, addy )
			lschbox:SetSize( 200, 20 )
			lschbox:SetText("Surpress engine lightning")
			lschbox:SetValue( 0 )
			lschbox.OnChange = function()
				data.surpresslightning = lschbox:GetChecked()
			end
			
			addy = addy + 22
			
			local matlabel = vgui.Create( "DLabel", panel )
			matlabel:SetPos( 5, addy )
			matlabel:SetSize( 80, 20 )
			matlabel:SetText( "Material:" )

			local mattext = vgui.Create("DTextEntry", panel )
			mattext:SetPos( 90, addy )
			mattext:SetSize( 160, 20 )
			mattext:SetMultiline(false)
			mattext:SetText( "" )
			mattext.OnTextChanged = function()
				local newmat = mattext:GetValue()
				if file.Exists ("../materials/"..newmat..".vmt") then
					data.material = newmat
				else
					data.material = ""
				end
			end
			
			return panel
			
		end
		
		/*** Sprite panel for adjusting sprites ***
		Name:
		Sprite:
		Bone name:
		Translation x / y / z
		Sprite x / y size
		Color
		*/
		local function CreateSpritePanel( name, x, y )
			
			local data = v_models[name]
			
			// default data
			data.type = "Sprite"
			data.sprite = ""
			data.bone = ""
			data.pos = Vector(0,0,0)
			data.size = { x = 1, y = 1 }
			data.color = Color(255,255,255,255)
			data.nocull = true
			data.additive = true
			data.vertexalpha = true
			data.vertexcolor = true
			data.ignorez = false
			
			local panel = vgui.Create( "DPanel", pmodels )
			panel:SetPos( x, y )
			panel:SetSize( 260, pmodels:GetTall() - panely - 60 )
			panel:SetPaintBackground( true )
			panel.Paint = function() surface.SetDrawColor( 70, 70, 70, 255 ) surface.DrawRect( 0, 0, panel:GetWide(), panel:GetTall() ) end
			
			local addy = 5
			
			local pnmlabel = vgui.Create( "DLabel", panel )
			pnmlabel:SetPos( 5, addy )
			pnmlabel:SetSize( 80, 20 )
			pnmlabel:SetText( "Name:" )
			
			local pnmtext = vgui.Create( "DTextEntry", panel )
			pnmtext:SetPos( 60, addy )
			pnmtext:SetSize( 190, 20 )
			pnmtext:SetMultiline(false)
			pnmtext:SetText( name )
			pnmtext:SetEditable( false )
			
			addy = addy + 25
			
			/*
			The bones
			*/
			local pmolabel = vgui.Create( "DLabel", panel )
			pmolabel:SetPos( 5, addy )
			pmolabel:SetSize( 80, 20 )
			pmolabel:SetText( "Sprite:" )
			
			local pmmtext = vgui.Create( "DTextEntry", panel )
			pmmtext:SetPos( 60, addy )
			pmmtext:SetSize( 190, 20 )
			pmmtext:SetMultiline(false)
			pmmtext:SetText( "" )
			pmmtext.OnTextChanged = function()
				local newsprite = pmmtext:GetValue()
				if file.Exists ("../materials/"..newsprite..".vmt") then
					data.sprite = newsprite
				end
			end

			addy = addy + 25
			
			local pbonelabel = vgui.Create( "DLabel", panel )
			pbonelabel:SetPos( 5, addy )
			pbonelabel:SetSize( 80, 20 )
			pbonelabel:SetText( "Bone:" )
			
			local bonebox = vgui.Create( "DMultiChoice", panel )
			bonebox:SetPos( 60, addy )
			bonebox:SetSize( 190, 20 )
			bonebox:SetEditable( false )
			bonebox:SetText( "" )
			bonebox.OnSelect = function( p, index, value )
				data.bone = value
			end
			
			local option = PopulateBoneList( bonebox )
			if (option) then 
				bonebox:ChooseOptionID(1)
			end
			
			addy = addy + 25
			
			// **************  Pos  **************
			
			local trlabel = vgui.Create( "DLabel", panel )
			trlabel:SetPos( 5, addy )
			trlabel:SetSize( 80, 20 )
			trlabel:SetText( "Pos x/y/z:" )
			
			local mxwang = vgui.Create( "DNumberWang", panel )
			mxwang:SetPos( 90, addy )
			mxwang:SetSize( 50, 20 )
			mxwang:SetMin( -20 )
			mxwang:SetMax( 20 )
			mxwang:SetDecimals( 3 )
			mxwang.OnValueChanged = function( p, value ) data.pos.x = tonumber(value) end

			local mywang = vgui.Create( "DNumberWang", panel )
			mywang:SetPos( 145, addy )
			mywang:SetSize( 50, 20 )
			mywang:SetMin( -20 )
			mywang:SetMax( 20 )
			mywang:SetDecimals( 3 )
			mywang.OnValueChanged = function( p, value ) data.pos.y = tonumber(value) end

			local mzwang = vgui.Create( "DNumberWang", panel )
			mzwang:SetPos( 200, addy )
			mzwang:SetSize( 50, 20 )
			mzwang:SetMin( -20 )
			mzwang:SetMax( 20 )
			mzwang:SetDecimals( 3 )
			mzwang.OnValueChanged = function( p, value ) data.pos.z = tonumber(value) end
			
			addy = addy + 25
			
			// **************  Size  **************
			
			local sizelabel = vgui.Create( "DLabel", panel )
			sizelabel:SetPos( 5, addy )
			sizelabel:SetSize( 80, 20 )
			sizelabel:SetText( "Size (xy)/y:" )
			
			local msxwang = vgui.Create( "DNumberWang", panel )
			msxwang:SetPos( 90, addy-2 )
			msxwang:SetSize( 52, 24 )
			msxwang:SetMin( 0.1 )
			msxwang:SetMax( 10 )
			msxwang:SetDecimals( 3 )

			local msywang = vgui.Create( "DNumberWang", panel )
			msywang:SetPos( 145, addy )
			msywang:SetSize( 50, 20 )
			msywang:SetMin( 0.1 )
			msywang:SetMax( 10 )
			msywang:SetDecimals( 3 )
			msywang.OnValueChanged = function( p, value ) data.size.y = tonumber(value) end
			
			msxwang.OnValueChanged = function( p, value )
				// make the x numberwang set the total size
				msywang:SetValue( value )
				data.size.x = tonumber(value)
			end
			msxwang:SetValue(1)

			addy = addy + 25
			
			local collabel = vgui.Create( "DLabel", panel )
			collabel:SetPos( 5, addy )
			collabel:SetSize( 80, 20 )
			collabel:SetText( "Color r/g/b/a:" )
			
			local colrwang = vgui.Create( "DNumberWang", panel )
			colrwang:SetPos( 90, addy )
			colrwang:SetSize( 38, 20 )
			colrwang:SetMin( 0 )
			colrwang:SetMax( 255 )
			colrwang:SetDecimals( 0 )
			colrwang.OnValueChanged = function( p, value ) data.color.r = tonumber(value) end
			colrwang:SetValue(255)
			
			local colgwang = vgui.Create( "DNumberWang", panel )
			colgwang:SetPos( 130, addy )
			colgwang:SetSize( 38, 20 )
			colgwang:SetMin( 0 )
			colgwang:SetMax( 255 )
			colgwang:SetDecimals( 0 )
			colgwang.OnValueChanged = function( p, value ) data.color.g = tonumber(value) end
			colgwang:SetValue(255)
			
			local colbwang = vgui.Create( "DNumberWang", panel )
			colbwang:SetPos( 170, addy )
			colbwang:SetSize( 38, 20 )
			colbwang:SetMin( 0 )
			colbwang:SetMax( 255 )
			colbwang:SetDecimals( 0 )
			colbwang.OnValueChanged = function( p, value ) data.color.b = tonumber(value) end
			colbwang:SetValue(255)
			
			local colawang = vgui.Create( "DNumberWang", panel )
			colawang:SetPos( 210, addy )
			colawang:SetSize( 40, 20 )
			colawang:SetMin( 0 )
			colawang:SetMax( 255 )
			colawang:SetDecimals( 0 )
			colawang.OnValueChanged = function( p, value ) data.color.a = tonumber(value) end
			colawang:SetValue(255)
			
			addy = addy + 25
			
			local ncchbox = vgui.Create( "DCheckBoxLabel", panel )
			ncchbox:SetPos( 5, addy )
			ncchbox:SetSize( 120, 20 )
			ncchbox:SetText("$nocull")
			ncchbox:SetValue( 1 )
			ncchbox.OnChange = function()
				data.nocull = ncchbox:GetChecked()
				data.spriteMaterial = nil // dump old material
			end
			
			local adchbox = vgui.Create( "DCheckBoxLabel", panel )
			adchbox:SetPos( 140, addy )
			adchbox:SetSize( 120, 20 )
			adchbox:SetText("$additive")
			adchbox:SetValue( 1 )
			adchbox.OnChange = function()
				data.additive = adchbox:GetChecked()
				data.spriteMaterial = nil // dump old material
			end
			
			addy = addy + 22
			
			local vtachbox = vgui.Create( "DCheckBoxLabel", panel )
			vtachbox:SetPos( 5, addy )
			vtachbox:SetSize( 120, 20 )
			vtachbox:SetText("$vertexalpha")
			vtachbox:SetValue( 1 )
			vtachbox.OnChange = function()
				data.vertexalpha = vtachbox:GetChecked()
				data.spriteMaterial = nil // dump old material
			end
			
			local vtcchbox = vgui.Create( "DCheckBoxLabel", panel )
			vtcchbox:SetPos( 140, addy )
			vtcchbox:SetSize( 120, 20 )
			vtcchbox:SetText("$vertexcolor")
			vtcchbox:SetValue( 1 )
			vtcchbox.OnChange = function()
				data.vertexcolor = vtcchbox:GetChecked()
				data.spriteMaterial = nil // dump old material
			end
			
			addy = addy + 22
			
			local izchbox = vgui.Create( "DCheckBoxLabel", panel )
			izchbox:SetPos( 5, addy )
			izchbox:SetSize( 120, 20 )
			izchbox:SetText("$ignorez")
			izchbox:SetValue( 0 )
			izchbox.OnChange = function()
				data.ignorez = izchbox:GetChecked()
				data.spriteMaterial = nil // dump old material
			end
			
			return panel
			
		end
		
		// cache the created panels
		local lastVisible = ""
		
		mlist.OnRowSelected = function( panel, line )
			local name = mlist:GetLine(line):GetValue(1)
			local ptype = mlist:GetLine(line):GetValue(2)
			if (!v_panelCache[name]) then
				if (ptype == "Model") then
					v_panelCache[name] = CreateModelPanel( name, 10, panely )
				elseif (ptype == "Sprite") then
					v_panelCache[name] = CreateSpritePanel( name, 10, panely )
				else
					Error("wtf are u doing")
				end
			end
			
			if (v_panelCache[lastVisible]) then
				v_panelCache[lastVisible]:SetVisible(false)
			end
			v_panelCache[name]:SetVisible(true)
			
			lastVisible = name
		end
		
		local prtbtn = vgui.Create( "DButton", pmodels)
		prtbtn:SetPos( 10, 465 )
		prtbtn:SetSize( 260, 20 )
		prtbtn:SetText("Print view model table to console")
		prtbtn.DoClick = function()
			MsgN("*********************************************")
			MsgN(GetVModelsText())
			MsgN("*********************************************")
			LocalPlayer():ChatPrint("Code printed to console!")
		end
		
		local pctbtn = vgui.Create( "DButton", pmodels)
		pctbtn:SetPos( 10, 490 )
		pctbtn:SetSize( 260, 20 )
		pctbtn:SetText("Copy view model table to clipboard")
		pctbtn.DoClick = function()
			SetClipboardText(GetVModelsText())
			LocalPlayer():ChatPrint("Code copied to clipboard!")
		end
		
		// remove a line
		rmbtn.DoClick = function()
			local line = mlist:GetSelectedLine()
			if (line) then
				local name = mlist:GetLine(line):GetValue(1)
				v_models[name] = nil
				// clear from panel cache
				if (v_panelCache[name]) then 
					v_panelCache[name]:Remove()
					v_panelCache[name] = nil
				end
				mlist:RemoveLine(line)
			end
		end
		
		/*********************
			World models page
		*********************/
		addy = 10
		
		local wmlabel = vgui.Create( "DLabel", pwmodels )
		wmlabel:SetPos( 10, addy )
		wmlabel:SetSize( 260, 20 )
		wmlabel:SetText( "Unavailable until Garry lets us change SWEP wmodels" )
		
		return f

	end

	function SWEP:OpenMenu()
		if (!self.Frame) then
			self.Frame = CreateMenu()
		end
		
		self.Frame:SetVisible(true)
		self.Frame:MakePopup()

	end
	
	function SWEP:CleanMenu()
		print("Removed frame")
		if (!self.Frame) then return end
		self.Frame:Remove()
		self.Frame = nil
	end
	
	/***************************
		Third person view
	***************************/
	function ThirdPersonCalcView(pl, pos, angles, fov)

		if (!useThirdPerson) then return end

		local view = {}
		view.origin = pos + ((pl:GetAngles()+thirdPersonAngle):Forward()*100)
		view.angles = (pos - view.origin):Angle()
		view.fov = fov
	 
		return view
	end
	 
	hook.Add("CalcView", "ThirdPersonCalcView", ThirdPersonCalcView)
	 
	hook.Add("ShouldDrawLocalPlayer", "ThirdPerson", function(pl)
			return useThirdPerson
	end)

end